ByChunks <- function(fun,
                     chunks = list(dataset = 1, member = 1, sdate = 1, 
                                   ftime = 1,   lat = 1,    lon = 1), 
                     var, exp, obs, sdates, ...,
                     silent = FALSE, debug = FALSE){
  library(s2dverification)
  MergeArrays <- easyNCDF:::.MergeArrays

  # Check fun
  if (!is.function(fun)) {
    stop("Parameter 'fun' must be a function.")
  }

  # Check silent
  if (!is.logical(silent)) {
    stop("Parameter 'silent' must be logical.")
  }

  # Check debug
  if (!is.logical(debug)) {
    stop("Parameter 'debug' must be logical.")
  }
  if (silent) {
    debug <- FALSE
  }

  # Check chunks
  if (!is.list(chunks)) {
    stop("Parameter 'chunks' must be a named list.")
  }
  if (is.null(names(chunks))) {
    stop("Parameter 'chunks' must be a named list.")
  }
  new_chunks <- list(dataset = 1, member = 1, sdate = 1, 
                     ftime = 1,   lat = 1,    lon = 1)
  if (any(!(names(chunks) %in% names(new_chunks)))) {
    stop("All names in parameter 'chunks' must be one of 'dataset', 'member', ",
         "'sdate', 'ftime', 'lat' or 'lon'.")
  }
  if (any(!(((unlist(chunks) %% 1) == 0) | (unlist(chunks) == 'all')))) {
    stop("All values in parameter 'chunks' must take a numeric value or 'all'.")
  }
  if (any(unlist(chunks) < 1)) {
    stop("All values in parameter 'chunks' must be > 1.")
  }
  new_chunks[names(chunks)] <- chunks
  chunks <- new_chunks

  # Check no chunking along members requested
  if (chunks[['member']] != 1) {
    stop("Chunking along members not available yet.")
  }

  # Discover coordinates
  s2dverification:::.message(paste("Discovering grid size..."))
  data_lonlat <- Load(var, exp[1], obs[1], 
                      sdates[1], leadtimemax = 1, 
                      nmember = 1, nmemberobs = 1, 
                      silent = !debug, ...)
  lons <- data_lonlat$lon
  lats <- data_lonlat$lat
  s2dverification:::.message(paste("Discovering forecast time steps..."))
  data_ftime <- Load(var, exp[1], obs[1], sdates[1], 
                     nmember = 1, nmemberobs = 1, 
                     lonmin = lons[1], lonmax = lons[1],
                     latmin = lats[1], latmax = lats[1], 
                     silent = !debug, ...)
  s2dverification:::.message(paste("Discovering ensemble members..."))
  data_members <- Load(var, exp[1], obs[1], 
                       sdates[1], leadtimemax = 1,
                       lonmin = lons[1], lonmax = lons[1],
                       latmin = lats[1], latmax = lats[1], 
                       silent = !debug, ...)
  if (!is.null(exp)) {
    nftime <- dim(data_ftime$mod)['ftime']
    nmemb_exp <- dim(data_ftime$mod)['ftime']
  } else {
    nftime <- dim(data_ftime$obs)['ftime']
    nmemb_obs <- dim(data_ftime$obs)['ftime']
  }

  # Replace 'all's
  if (chunks[['dataset']] == 'all') {
    chunks[['dataset']] <- length(exp)
  }
  if (chunks[['member']] == 'all') {
    chunks[['dataset']] <- nmemb_exp
  }
  if (chunks[['sdate']] == 'all') {
    chunks[['sdate']] <- length(sdates)
  }
  if (chunks[['ftime']] == 'all') {
    chunks[['ftime']] <- nftime
  }
  if (chunks[['lat']] == 'all') {
    chunks[['lat']] <- length(lats)
  }
  if (chunks[['lon']] == 'all') {
    chunks[['lon']] <- length(lons)
  }

  # Empty array to collect results
  result <- list(mod = NULL, obs = NULL)

  chunk_counter <- 0
  if (chunks[['dataset']] > length(exp)) {
    warning("Too many 'dataset' chunks requested. Reducing from ",
            chunks[['dataset']], " to ", length(exp), ".")
    chunks[['dataset']] <- length(exp)
  }
  datasets_per_chunk <- ceiling(length(exp) / chunks[['dataset']])
  dataset_index <- 1
  result <- list(mod = NULL, obs = NULL)
  for (dataset in 1:chunks[['dataset']]) {
    dataset_min <- (1:length(exp))[dataset_index]
    dataset_max <- (1:length(exp))[min(length(exp), dataset_index + datasets_per_chunk - 1)]

    if (chunks[['member']] > nmemb_exp) {
      warning("Too many 'member' chunks requested. Reducing from ",
              chunks[['member']], " to ", nmemb_exp, ".")
      chunks[['member']] <- nmemb_exp
    }
    members_per_chunk <- ceiling(nmemb_exp / chunks[['member']])
    member_index <- 1
    res_member <- list(mod = NULL, obs = NULL)
    for (member in 1:chunks[['member']]) {
      member_min <- (1:nmemb_exp)[member_index]
      member_max <- (1:nmemb_exp)[min(nmemb_exp, member_index + members_per_chunk - 1)]

      if (chunks[['sdate']] > length(sdates)) {
        warning("Too many 'sdate' chunks requested. Reducing from ",
                chunks[['sdate']], " to ", length(sdates), ".")
                chunks[['sdate']] <- length(sdate)
      }
      sdates_per_chunk <- ceiling(length(sdates) / chunks[['sdate']])
      sdate_index <- 1
      res_sdate <- list(mod = NULL, obs = NULL)
      for (sdate in 1:chunks[['sdate']]) {
        sdate_min <- (1:length(sdates))[sdate_index]
        sdate_max <- (1:length(sdates))[min(length(sdates), sdate_index + sdates_per_chunk - 1)]    

        if (chunks[['ftime']] > nftime) {
          warning("Too many 'ftime' chunks requested. Reducing from ",
                  chunks[['ftime']], " to ", nftime, ".")
          chunks[['ftime']] <- nftime
        }
        ftimes_per_chunk <- ceiling(nftime / chunks[['ftime']])
        ftime_index <- 1
        res_ftime <- list(mod = NULL, obs = NULL)
        for (ftime in 1:chunks[['ftime']]) {
          ftime_min <- (1:nftime)[ftime_index]
          ftime_max <- (1:nftime)[min(nftime, ftime_index + ftimes_per_chunk - 1)]

          if (chunks[['lat']] > length(lats)) {
            warning("Too many 'lat' chunks requested. Reducing from ",
                    chunks[['lat']], " to ", length(lats), ".")
            chunks[['lat']] <- length(lats)
          }
          lats_per_chunk <- ceiling(length(lats) / chunks[['lat']])
          lat_index <- 1
          res_lat <- list(mod = NULL, obs = NULL)
          for (lat in 1:chunks[['lat']]) {
            lat_max <- lats[lat_index]
            lat_min <- lats[min(length(lats), lat_index + lats_per_chunk - 1)]

            if (chunks[['lon']] > length(lons)) {
              warning("Too many 'lon' chunks requested. Reducing from ",
                      chunks[['lon']], " to ", length(lons), ".")
              chunks[['lon']] <- length(lons)
            }
            lons_per_chunk <- ceiling(length(lons) / chunks[['lon']])
            lon_index <- 1
            res_lon <- list(mod = NULL, obs = NULL)
            for (lon in 1:chunks[['lon']]) {
              lon_min <- lons[lon_index]
              lon_max <- lons[min(length(lons), lon_index + lons_per_chunk - 1)]

              if (!silent) {
                s2dverification:::.message(paste("Loading chunk", chunk_counter + 1, 
                                           "out of", prod(unlist(chunks)), "..."))
              }

              chunk_counter <- chunk_counter + 1

              data <- Load(var, exp[dataset_min:dataset_max], obs,
                           sdates[sdate_min:sdate_max], 
                           leadtimemin = ftime_min, leadtimemax = ftime_max,
                           latmin = lat_min, latmax = lat_max,
                           lonmin = lon_min, lonmax = lon_max, 
                           silent = !debug, ...)

              if (!silent) {
                s2dverification:::.message(paste("Processing..."))
              }

              res <- fun(data)

              rm(data)
              gc()

              if (chunks[['lon']] > 1) {
                res_lon$mod <- MergeArrays(res_lon$mod, res$mod, 'lon')
                res_lon$obs <- MergeArrays(res_lon$obs, res$obs, 'lon')
              } else {
                res_lon <- res
              }
              rm(res)
              gc()
              lon_index <- lon_index + lons_per_chunk              
            }
            if (chunks[['lat']] > 1) {
              res_lat$mod <- MergeArrays(res_lat$mod, res_lon$mod, 'lat')
              res_lat$obs <- MergeArrays(res_lat$obs, res_lon$obs, 'lat')
            } else {
              res_lat <- res_lon
            }
            rm(res_lon)
            gc()
            lat_index <- lat_index + lats_per_chunk 
          }
          if (chunks[['ftime']] > 1) {
            res_ftime$mod <- MergeArrays(res_ftime$mod, res_lat$mod, 'ftime')
            res_ftime$obs <- MergeArrays(res_ftime$obs, res_lat$obs, 'ftime')
          } else {
            res_ftime <- res_lat
          }
          rm(res_lat)
          gc()
          ftime_index <- ftime_index + ftimes_per_chunk 
        }
        if (chunks[['sdate']] > 1) {
          res_sdate$mod <- MergeArrays(res_sdate$mod, res_ftime$mod, 'sdate')
          res_sdate$obs <- MergeArrays(res_sdate$obs, res_ftime$obs, 'sdate')
        } else {
          res_sdate <- res_ftime
        }
        rm(res_ftime)
        gc()
        sdate_index <- sdate_index + sdates_per_chunk 
      }
      if (chunks[['member']] > 1) {
        res_member$mod <- MergeArrays(res_member$mod, res_sdate$mod, 'member')
        res_member$obs <- MergeArrays(res_member$obs, res_sdate$obs, 'member')
      } else {
        res_member <- res_sdate
      }
      rm(res_sdate)
      gc()
      member_index <- member_index + members_per_chunk 
    }
    if (chunks[['dataset']] > 1) {
      result$mod <- MergeArrays(res_dataset$mod, res_member$mod, 'dataset')
      result$obs <- MergeArrays(res_dataset$obs, res_member$obs, 'dataset')
    } else {
      result <- res_member
    }
    rm(res_member)
    gc()
    dataset_index <- dataset_index + datasets_per_chunk 
  }
  result
}
